==> nui_core-0.0.127.tar.gz: lib/data/nui_ent.dart <==
import 'dart:convert';
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';

typedef EntFieldSetter<T> = Object Function(T data, dynamic value);
typedef EntFieldGetter<T> = dynamic Function(T data);
typedef EntityGetter<T> = NUIEntMapper<T> Function();

enum NUIEntDisplayType{
  LABEL,
  NAME,
  DATE,
  TIME,
  CURRENCY,
  DISTANCE,
  ADDRESS,
  IMAGE_URL,
  IMAGE_FILE,
  COLOR
}

enum NUIEntType{
  STRING,
  INTEGER,
  DOUBLE,
  DATE,
  BOOLEAN,
  OBJECT,
  MAP,
  LIST_INT,
  LIST_STRING,
  LIST_DOUBLE,
  LIST_OBJECT,
  LIST_MAP,
}

NUIEntType valueOfEntType(String value){
  switch(value){
    case "STRING" : return NUIEntType.STRING;
    case "INTEGER" : return NUIEntType.INTEGER;
    case "DOUBLE" : return NUIEntType.DOUBLE;
    case "DATE" : return NUIEntType.DATE;
    case "BOOLEAN" : return NUIEntType.BOOLEAN;
    case "OBJECT" : return NUIEntType.OBJECT;
    case "MAP" : return NUIEntType.MAP;
    case "LIST INT" : return NUIEntType.LIST_INT;
    case "LIST STRING" : return NUIEntType.LIST_STRING;
    case "LIST DOUBLE" : return NUIEntType.LIST_DOUBLE;
    case "LIST OBJECT" : return NUIEntType.LIST_OBJECT;
    case "LIST MAP" : return NUIEntType.LIST_MAP;
    default: return null;
  }
}

extension NUIEntTypeExt on NUIEntType{
  String value(){
    switch(this){
      case NUIEntType.STRING : return "STRING";
      case NUIEntType.INTEGER : return "INTEGER";
      case NUIEntType.DOUBLE : return "DOUBLE";
      case NUIEntType.DATE : return "DATE";
      case NUIEntType.BOOLEAN : return "BOOLEAN";
      case NUIEntType.OBJECT : return "OBJECT";
      case NUIEntType.MAP : return "MAP";
      case NUIEntType.LIST_INT : return "LIST INT";
      case NUIEntType.LIST_STRING : return "LIST STRING";
      case NUIEntType.LIST_DOUBLE : return "LIST DOUBLE";
      case NUIEntType.LIST_OBJECT : return "LIST OBJECT";
      case NUIEntType.LIST_MAP : return "LIST MAP";
      default: return null;
    }
  }
}

class NUIFormatting{
  NUIFormatting._builder();
  static const String TWO_DECIMAL = "TWO DECIMAL";
  static const String ONE_DECIMAL = "ONE DECIMAL";
  static const String ROUNDED = "ROUNDED";
}

D asObject<D>(NUIEntMapper<D> entity, dynamic value){
  if(value == null) return null;
  if(value is Map){
    return entity.toEntity(value);
  }
  return entity.toEntity(jsonDecode(value));
}

Map<String, dynamic> asMap(dynamic value){
  try{
    return jsonDecode(value);
  }catch(e){
    return null;
  }
}

List<E> asListOfObjects<E>(NUIEntMapper<E> entity, dynamic value){
  if(value == null) return List();
  var endValue = value;
  if(value is String){
    endValue = jsonDecode(value);
  }
  List<E> list = (endValue as List).map((i) =>
      entity.toEntity(i)).toList();
  return list;
}

List<T> asList<T>(dynamic value){
  if(value == null) return null;
  var endValue = value;
  if(value is String){
    endValue = jsonDecode(value);
  }
  return List<T>.from(endValue);
}

DateTime asDateTime(dynamic value){
  if(value == null) return null;
  if(value is String){
    return stringToDate(value);
  }
  return DateTime.now();
}

class NUIUIEntBinding{
  NUIEntDisplayType displayType;
  String format;
  bool ellipsis;
}

abstract class NUIEnt<T>{
  NUIEntMapper<T> mapper();

  T fromJson(Map<String, dynamic> json){
    return mapper().toEntity(json);
  }

  Map<String, dynamic> toMap(){
    Map<String, dynamic> map = Map();
    for(NUIEntField field in mapper().fields()){
      map[field.name] = field.runGetter(this);
    }
    return map;
  }

  Map<String, dynamic> toMapQuery(){
    final json = toMap();
    for(String key in json.keys){
      final val = json[key];
      if(val is NUIEnt){
        json[key] = jsonEncode(val);
      }
      if(val is Map){
        json[key] = jsonEncode(val);
      }
      if(val is List<NUIEnt>){
        json[key] = jsonEncode(val);
      }
      if(val is List){
        json[key] = jsonEncode(val);
      }
      if(val is DateTime){
        json[key] = dateToString(val);
      }
    }
    return json;
  }

  Map<String, dynamic> toJson(){
    final json = toMap();
    for(String key in json.keys){
      final val = json[key];
      if(val is DateTime){
        json[key] = dateToString(val);
      }
      if(val is NUIEnt){
        json[key] = val.toJson();
      }
      if(val is List<NUIEnt>){
        final mapList = List<Map<String, dynamic>>();
        for(NUIEnt e in val){
          mapList.add(e.toJson());
        }
        json[key] = mapList;
      }
    }
    return json;
  }
}

class NUIEntField<T> {
  NUIEntField({@required this.name, @required this.type, @required this.setter, @required this.getter});

  String name;
  NUIEntType type;
  final EntFieldSetter<T> setter;
  final EntFieldGetter<T> getter;

  void runSetter(T data, dynamic value){
    setter(data, value);
  }

  dynamic runGetter(T data){
    return getter(data);
  }
}

abstract class NUIEntMapper<T>{
  T newInstance();

  String entityName();
  List<NUIEntField> fields();

  T toEntity(Map<String, dynamic> map){
    T data = newInstance();
    for(NUIEntField field in fields()){
//      if(field.type == NUIEntType.OBJECT && field.entity != null){
//        field.runSetter(data, toObject(field.entity, map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_OBJECT && field.entity != null){
//        field.runSetter(data, toListOfObjects(field.getTargetEntity(), map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_INT){
//        field.runSetter(data, toList<int>(map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_STRING){
//        field.runSetter(data, toList<String>(map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_DOUBLE){
//        field.runSetter(data, asListOfObjects<double>(field.entity, map[field.name]));
//      }
      try{
        if(field.type == NUIEntType.DATE){
          field.runSetter(data, asDateTime(map[field.name]));
        }
        else if(field.type == NUIEntType.MAP){
          field.runSetter(data, asMap(map[field.name]));
        }
        else if(field.type == NUIEntType.INTEGER && map[field.name] != null && map[field.name] is double){
          field.runSetter(data, (map[field.name] as double).toInt());
        }
        else if(field.type == NUIEntType.DOUBLE && map[field.name] != null && map[field.name] is int){
          field.runSetter(data, (map[field.name] as int).toDouble());
        }
        else {
          field.runSetter(data, map[field.name]);
        }
      }catch(e){
        log("NUIJson -> Failed to read field : ${field.name} of type : ${field.type} with exception : ${e.toString()}");
      }

    }
    return data;
  }
}

==> nui_core-0.0.150.tar.gz: lib/data/nui_ent.dart <==
import 'dart:convert';
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';

typedef EntFieldSetter<T> = Object Function(T data, dynamic value);
typedef EntFieldGetter<T> = dynamic Function(T data);
typedef EntityGetter<T> = NUIEntMapper<T> Function();

enum NUIEntDisplayType{
  LABEL,
  NAME,
  DATE,
  TIME,
  CURRENCY,
  DISTANCE,
  ADDRESS,
  IMAGE_URL,
  IMAGE_FILE,
  COLOR
}

enum NUIEntType{
  STRING,
  INTEGER,
  DOUBLE,
  DATE,
  BOOLEAN,
  OBJECT,
  MAP,
  LIST_INT,
  LIST_STRING,
  LIST_DOUBLE,
  LIST_OBJECT,
  LIST_MAP,
}

NUIEntType valueOfEntType(String value){
  switch(value){
    case "STRING" : return NUIEntType.STRING;
    case "INTEGER" : return NUIEntType.INTEGER;
    case "DOUBLE" : return NUIEntType.DOUBLE;
    case "DATE" : return NUIEntType.DATE;
    case "BOOLEAN" : return NUIEntType.BOOLEAN;
    case "OBJECT" : return NUIEntType.OBJECT;
    case "MAP" : return NUIEntType.MAP;
    case "LIST INT" : return NUIEntType.LIST_INT;
    case "LIST STRING" : return NUIEntType.LIST_STRING;
    case "LIST DOUBLE" : return NUIEntType.LIST_DOUBLE;
    case "LIST OBJECT" : return NUIEntType.LIST_OBJECT;
    case "LIST MAP" : return NUIEntType.LIST_MAP;
    default: return null;
  }
}

extension NUIEntTypeExt on NUIEntType{
  String value(){
    switch(this){
      case NUIEntType.STRING : return "STRING";
      case NUIEntType.INTEGER : return "INTEGER";
      case NUIEntType.DOUBLE : return "DOUBLE";
      case NUIEntType.DATE : return "DATE";
      case NUIEntType.BOOLEAN : return "BOOLEAN";
      case NUIEntType.OBJECT : return "OBJECT";
      case NUIEntType.MAP : return "MAP";
      case NUIEntType.LIST_INT : return "LIST INT";
      case NUIEntType.LIST_STRING : return "LIST STRING";
      case NUIEntType.LIST_DOUBLE : return "LIST DOUBLE";
      case NUIEntType.LIST_OBJECT : return "LIST OBJECT";
      case NUIEntType.LIST_MAP : return "LIST MAP";
      default: return null;
    }
  }
}

class NUIFormatting{
  NUIFormatting._builder();
  static const String TWO_DECIMAL = "TWO DECIMAL";
  static const String ONE_DECIMAL = "ONE DECIMAL";
  static const String ROUNDED = "ROUNDED";
}

D asObject<D>(NUIEntMapper<D> entity, dynamic value){
  if(value == null) return null;
  if(value is Map){
    return entity.toEntity(value);
  }
  return entity.toEntity(jsonDecode(value));
}

Map<String, dynamic> asMap(dynamic value){
  try{
    return jsonDecode(value);
  }catch(e){
    return null;
  }
}

Map<String, T> asMapOfType<T>(dynamic value){
  final map = asMap(value);
  final mapObject = Map<String, T>();
  map.forEach((key, value) {
    try{
      T data = value as T;
      mapObject[key] = data;
    }catch(e){
      logNUI("NUIEnt", "Failed to map data of ${T.runtimeType} from map");
    }
  });
  return mapObject;
}

Map<String, T> asMapOfObject<T>(NUIEntMapper<T> entity, dynamic value){
  final map = asMap(value);
  final mapObject = Map<String, T>();
  map.forEach((key, value) {
    try{
      T objectData = entity.toEntity(value);
      mapObject[key] = objectData;
    }catch(e){
      logNUI("NUIEnt", "Failed to map object of ${entity.entityName()} from map");
    }
  });
  return mapObject;
}

List<E> asListOfObjects<E>(NUIEntMapper<E> entity, dynamic value){
  if(value == null) return [];
  var endValue = value;
  if(value is String){
    endValue = jsonDecode(value);
  }
  List<E> list = (endValue as List).map((i) =>
      entity.toEntity(i)).toList();
  return list;
}

List<T> asList<T>(dynamic value){
  if(value == null) return null;
  var endValue = value;
  if(value is String){
    endValue = jsonDecode(value);
  }
  return List<T>.from(endValue);
}

DateTime asDateTime(dynamic value){
  if(value == null) return null;
  if(value is String){
    return stringToDate(value);
  }
  return DateTime.now();
}

class NUIUIEntBinding{
  NUIEntDisplayType displayType;
  String format;
  bool ellipsis;
}

abstract class NUIEnt<T>{
  NUIEntMapper<T> mapper();

  T fromJson(Map<String, dynamic> json){
    return mapper().toEntity(json);
  }

  Map<String, dynamic> toMap(){
    Map<String, dynamic> map = Map();
    for(NUIEntField field in mapper().fields()){
      map[field.name] = field.runGetter(this);
    }
    return map;
  }

  Map<String, dynamic> toMapQuery(){
    final json = toMap();
    for(String key in json.keys){
      final val = json[key];
      if(val is NUIEnt){
        json[key] = jsonEncode(val);
      }
      if(val is Map){
        json[key] = jsonEncode(val);
      }
      if(val is List<NUIEnt>){
        json[key] = jsonEncode(val);
      }
      if(val is List){
        json[key] = jsonEncode(val);
      }
      if(val is DateTime){
        json[key] = dateToString(val);
      }
    }
    return json;
  }

  Map<String, dynamic> toJson(){
    final json = toMap();
    for(String key in json.keys){
      final val = json[key];
      if(val is DateTime){
        json[key] = dateToString(val);
      }
      if(val is NUIEnt){
        json[key] = val.toJson();
      }
      if(val is List<NUIEnt>){
        final mapList = List<Map<String, dynamic>>();
        for(NUIEnt e in val){
          mapList.add(e.toJson());
        }
        json[key] = mapList;
      }
    }
    return json;
  }
}

class NUIEntField<T> {
  NUIEntField({@required this.name, @required this.type, @required this.setter, @required this.getter});

  String name;
  NUIEntType type;
  final EntFieldSetter<T> setter;
  final EntFieldGetter<T> getter;

  void runSetter(T data, dynamic value){
    setter(data, value);
  }

  dynamic runGetter(T data){
    return getter(data);
  }
}

abstract class NUIEntMapper<T>{
  T newInstance();

  String entityName();
  List<NUIEntField> fields();

  T toEntity(Map<String, dynamic> map){
    T data = newInstance();
    for(NUIEntField field in fields()){
//      if(field.type == NUIEntType.OBJECT && field.entity != null){
//        field.runSetter(data, toObject(field.entity, map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_OBJECT && field.entity != null){
//        field.runSetter(data, toListOfObjects(field.getTargetEntity(), map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_INT){
//        field.runSetter(data, toList<int>(map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_STRING){
//        field.runSetter(data, toList<String>(map[field.name]));
//      }
//      else if(field.type == NUIEntType.LIST_DOUBLE){
//        field.runSetter(data, asListOfObjects<double>(field.entity, map[field.name]));
//      }
      try{
        if(field.type == NUIEntType.DATE){
          field.runSetter(data, asDateTime(map[field.name]));
        }
        else if(field.type == NUIEntType.MAP){
          field.runSetter(data, asMap(map[field.name]));
        }
        else if(field.type == NUIEntType.INTEGER && map[field.name] != null && map[field.name] is double){
          field.runSetter(data, (map[field.name] as double).toInt());
        }
        else if(field.type == NUIEntType.DOUBLE && map[field.name] != null && map[field.name] is int){
          field.runSetter(data, (map[field.name] as int).toDouble());
        }
        else {
          field.runSetter(data, map[field.name]);
        }
      }catch(e){
        log("NUIJson -> Failed to read field : ${field.name} of type : ${field.type} with exception : ${e.toString()}");
      }

    }
    return data;
  }
}
